<?php

/**
 * WeEngine System
 *
 * (c) We7Team 2022 <https://www.w7.cc>
 *
 * This is not a free software
 * Using it under the license terms
 * visited https://www.w7.cc for more details
 */

namespace W7\Sdk\Module;

use W7\Sdk\Module\Exceptions\ApiException;
use W7\Sdk\Module\Exceptions\ApiHttpException;
use W7\Sdk\Module\Exceptions\ConnectException;
use W7\Sdk\Module\Exceptions\InvalidArgumentException;
use W7\Sdk\Module\Exceptions\RequestException;
use W7\Sdk\Module\Http\Client;
use W7\Sdk\Module\Interfaces\ResponseInterface;
use W7\Sdk\Module\Middlewares\RequestMiddlewares;
use W7\Sdk\Module\Middlewares\ResponseMiddlewares;
use W7\Sdk\Module\Pay\AliPay;
use W7\Sdk\Module\Pay\Wechat;
use W7\Sdk\Module\Support\Account;
use W7\Sdk\Module\Support\AccountRequest;
use W7\Sdk\Module\Support\ApiResponse;

class Api
{
    /** @var Client */
    protected $client;

    /** @var Wechat */
    protected $wechatPay;

    /** @var AliPay */
    protected $aliPay;

    /** @var AccountRequest */
    protected $app;

    /** @var string */
    protected $appId;

    /** @var string */
    protected $linkAppId;

    /** @var string */
    protected $appSecret;

    /** @var int  */
    protected $accountType;

    /**
     * @param string $app_id       应用关联的Appid
     * @param string $app_secret   应用关联AppSecret
     * @param string $link_app_id  应用关联App Id
     * @param int    $account_type 平台类型，使用
     * @param string $base_uri     基础通信URI
     *
     * @throws InvalidArgumentException
     */
    public function __construct(
        $app_id,
        $app_secret,
        $link_app_id = '',
        $account_type = Account::TYPE_NONE,
        $base_uri = 'https://api.w7.cc/w7api/mgw/w7_rangineapi/'
    ) {
        $this->appId       = $app_id;
        $this->appSecret   = $app_secret;
        $this->accountType = $account_type;
        $this->linkAppId   = $link_app_id;

        $this->client = new Client($base_uri, [
            'response_middlewares' => new ResponseMiddlewares(),
            'request_middlewares'  => new RequestMiddlewares($this->appId, $this->appSecret, $this->linkAppId, $this->accountType)
        ]);
    }

    /**
     * @return AccountRequest
     */
    public function app()
    {
        if (!isset($this->app)) {
            $this->app = new AccountRequest($this->client);
        }

        return $this->app;
    }

    /**
     * @param string $notify_url
     * @return Wechat
     */
    public function wechatPay($notify_url = '')
    {
        if (!isset($this->wechatPay)) {
            $this->wechatPay = new Wechat($this->client, $notify_url);
        }

        return $this->wechatPay;
    }

    /**
     * @param string $notify_url
     * @return AliPay
     */
    public function aliPay($notify_url = '')
    {
        if (!isset($this->aliPay)) {
            $this->aliPay = new AliPay($this->client, $notify_url);
        }

        return $this->aliPay;
    }

    /**
     * 获取关联下的平台列表
     *
     * @return ResponseInterface|ApiResponse
     *
     * @throws RequestException
     * @throws ConnectException
     * @throws ApiHttpException
     * @throws ApiException
     *
     * @noinspection PhpDocRedundantThrowsInspection
     * @noinspection PhpReturnDocTypeMismatchInspection
     */
    public function getAccountList()
    {
        return $this->client->post('open/api/account/list');
    }

    /**
     * 登录获取站点创始人Token
     *
     * @return ResponseInterface|ApiResponse
     *
     * @throws RequestException
     * @throws ConnectException
     * @throws ApiHttpException
     * @throws ApiException
     *
     * @noinspection PhpDocRedundantThrowsInspection
     * @noinspection PhpReturnDocTypeMismatchInspection
     */
    public function login()
    {
        return $this->client->post('open/api/login');
    }

    /**
     * 获取当前站点下的所有关联
     *
     * @return ResponseInterface|ApiResponse
     *
     * @throws RequestException
     * @throws ConnectException
     * @throws ApiHttpException
     * @throws ApiException
     *
     * @noinspection PhpDocRedundantThrowsInspection
     * @noinspection PhpReturnDocTypeMismatchInspection
     */
    public function getAllLink()
    {
        return $this->client->post('open/api/link/all');
    }
}
