<?php

/**
 * WeEngine System
 *
 * (c) We7Team 2022 <https://www.w7.cc>
 *
 * This is not a free software
 * Using it under the license terms
 * visited https://www.w7.cc for more details
 */

namespace W7\Sdk\Module\Http;

use W7\Sdk\Module\Exceptions\ConnectException;
use W7\Sdk\Module\Exceptions\RequestException;
use W7\Sdk\Module\Interfaces\RequestInterface;
use W7\Sdk\Module\Interfaces\ResponseInterface;
use W7\Sdk\Module\Support\Utils;

class CurlFactory
{
    /**
     * @var resource[]|\CurlHandle[]
     */
    private $handles = [];

    /**
     * @var int 保留在缓存中的空闲句柄总数
     */
    private $maxHandles;

    public function __construct($maxHandles)
    {
        $this->maxHandles = $maxHandles;
    }

    public function create(RequestInterface $request, array $options)
    {
        if (isset($options['curl']['body_as_string'])) {
            $options['_body_as_string'] = $options['curl']['body_as_string'];
            unset($options['curl']['body_as_string']);
        }

        $responseHandle          = new ResponseHandle();
        $responseHandle->request = $request;
        $responseHandle->options = $options;
        if (isset($options['response_middlewares'])) {
            $responseHandle->responseMiddlewares = $options['response_middlewares'];
        }
        $conf = $this->getDefaultConf($responseHandle);
        $this->applyMethod($responseHandle, $conf);
        $this->applyHandlerOptions($responseHandle, $conf);
        $this->applyHeaders($responseHandle, $conf);
        unset($conf['_headers']);

        // 从请求配置选项中添加处理程序选项
        if (isset($options['curl'])) {
            $conf = \array_replace($conf, $options['curl']);
        }

        $conf[\CURLOPT_HEADERFUNCTION] = $this->createHeaderFn($responseHandle);
        $responseHandle->handle        = $this->handles ? \array_pop($this->handles) : \curl_init();
        curl_setopt_array($responseHandle->handle, $conf);

        return $responseHandle;
    }

    /**
     * @param ResponseHandle $handle
     * @return \Closure
     */
    private function createHeaderFn(ResponseHandle $handle)
    {
        if (isset($easy->options['on_headers'])) {
            $onHeaders = $easy->options['on_headers'];

            if (!\is_callable($onHeaders)) {
                throw new \InvalidArgumentException('on_headers must be callable');
            }
        } else {
            $onHeaders = null;
        }

        return static function ($ch, $h) use (
            $onHeaders,
            $handle,
            &$startingResponse
        ) {
            $value = \trim($h);
            if ('' === $value) {
                $startingResponse = true;
                try {
                    $handle->createResponse();
                } catch (\Exception $e) {
                    $handle->createResponseException = $e;

                    return -1;
                }
                if (null !== $onHeaders) {
                    try {
                        $onHeaders($handle->response);
                    } catch (\Exception $e) {
                        // 将异常与句柄相关联，并通过返回0来触发curl头的写入错误。
                        $handle->onHeadersException = $e;

                        return -1;
                    }
                }
            } elseif ($startingResponse) {
                $startingResponse = false;
                $handle->headers  = [$value];
            } else {
                $handle->headers[] = $value;
            }

            return \strlen($h);
        };
    }

    private function applyHandlerOptions(ResponseHandle $handle, array &$conf)
    {
        $options = $handle->options;
        if (isset($options['verify'])) {
            if (false === $options['verify']) {
                unset($conf[\CURLOPT_CAINFO]);
                $conf[\CURLOPT_SSL_VERIFYHOST] = 0;
                $conf[\CURLOPT_SSL_VERIFYPEER] = false;
            } else {
                $conf[\CURLOPT_SSL_VERIFYHOST] = 2;
                $conf[\CURLOPT_SSL_VERIFYPEER] = true;
                if (\is_string($options['verify'])) {
                    // 如果文件/文件夹/链接路径无效或不存在，则抛出错误。
                    if (!\file_exists($options['verify'])) {
                        throw new \InvalidArgumentException("SSL CA bundle not found: {$options['verify']}");
                    }
                    // 如果是目录或指向目录的链接，则使用CURLOPT_CAPATH。
                    // 如果不是，它可能是一个文件，或者是一个到文件的链接，所以使用CURLOPT_CAINFO。
                    if (
                        \is_dir($options['verify']) || (
                            true === \is_link($options['verify']) && ($verifyLink = \readlink($options['verify'])) !== false && \is_dir($verifyLink)
                        )
                    ) {
                        $conf[\CURLOPT_CAPATH] = $options['verify'];
                    } else {
                        $conf[\CURLOPT_CAINFO] = $options['verify'];
                    }
                }
            }
        }

        if (!isset($options['curl'][\CURLOPT_ENCODING]) && !empty($options['decode_content'])) {
            $accept = $handle->request->getHeaderLine('Accept-Encoding');
            if ($accept) {
                $conf[\CURLOPT_ENCODING] = $accept;
            } else {
                // 空字符串启用所有可用的解码器，并隐式设置匹配的'Accept-Encoding'标头。
                $conf[\CURLOPT_ENCODING] = '';
                // 但是由于用户没有指定任何可接受的编码，我们需要用一个空的标头覆盖这个隐式标头。
                $conf[\CURLOPT_HTTPHEADER][] = 'Accept-Encoding:';
            }
        }

        if (!isset($options['sink'])) {
            // 如果没有设置接收器，则使用默认的临时流。
            $options['sink'] = Utils::tryFopen('php://temp', 'w+');
        }
        $sink = $options['sink'];
        if (!\is_string($sink)) {
            $sink = Utils::streamFor($sink);
        } elseif (!\is_dir(\dirname($sink))) {
            // 在curl失败之前，请确保该目录存在。
            throw new \RuntimeException(\sprintf('Directory %s does not exist for sink value of %s', \dirname($sink), $sink));
        } else {
            throw new \RuntimeException(\sprintf('Invalid sink value of %s', $sink));
        }
        $handle->sink                 = $sink;
        $conf[\CURLOPT_WRITEFUNCTION] = static function ($ch, $write) use ($sink) {
            return $sink->write($write);
        };

        $timeoutRequiresNoSignal = false;
        if (isset($options['timeout'])) {
            $timeoutRequiresNoSignal |= $options['timeout'] < 1;
            $conf[\CURLOPT_TIMEOUT_MS] = $options['timeout'] * 1000;
        }

        // CURL默认值是CURL_IPRESOLVE_WHATEVER，它将尝试IPv4，然后尝试IPv6。
        if (isset($options['force_ip_resolve'])) {
            if ('v4' === $options['force_ip_resolve']) {
                $conf[\CURLOPT_IPRESOLVE] = \CURL_IPRESOLVE_V4;
            } elseif ('v6' === $options['force_ip_resolve']) {
                $conf[\CURLOPT_IPRESOLVE] = \CURL_IPRESOLVE_V6;
            }
        }

        if (isset($options['connect_timeout'])) {
            $timeoutRequiresNoSignal |= $options['connect_timeout'] < 1;
            $conf[\CURLOPT_CONNECTTIMEOUT_MS] = $options['connect_timeout'] * 1000;
        }

        if ($timeoutRequiresNoSignal && 'WIN' !== \strtoupper(\substr(\PHP_OS, 0, 3))) {
            $conf[\CURLOPT_NOSIGNAL] = true;
        }

        if (isset($options['proxy'])) {
            if (!\is_array($options['proxy'])) {
                $conf[\CURLOPT_PROXY] = $options['proxy'];
            } else {
                $scheme = $handle->request->getUri()->getScheme();
                if (isset($options['proxy'][$scheme])) {
                    $host = $handle->request->getUri()->getHost();
                    if (isset($options['proxy']['no']) && Utils::isHostInNoProxy($host, $options['proxy']['no'])) {
                        unset($conf[\CURLOPT_PROXY]);
                    } else {
                        $conf[\CURLOPT_PROXY] = $options['proxy'][$scheme];
                    }
                }
            }
        }

        if (isset($options['crypto_method'])) {
            if (\STREAM_CRYPTO_METHOD_TLSv1_0_CLIENT === $options['crypto_method']) {
                if (!defined('CURL_SSLVERSION_TLSv1_0')) {
                    throw new \InvalidArgumentException('Invalid crypto_method request option: TLS 1.0 not supported by your version of cURL');
                }
                $conf[\CURLOPT_SSLVERSION] = \CURL_SSLVERSION_TLSv1_0;
            } elseif (\STREAM_CRYPTO_METHOD_TLSv1_1_CLIENT === $options['crypto_method']) {
                if (!defined('CURL_SSLVERSION_TLSv1_1')) {
                    throw new \InvalidArgumentException('Invalid crypto_method request option: TLS 1.1 not supported by your version of cURL');
                }
                $conf[\CURLOPT_SSLVERSION] = \CURL_SSLVERSION_TLSv1_1;
            } elseif (\STREAM_CRYPTO_METHOD_TLSv1_2_CLIENT === $options['crypto_method']) {
                if (!defined('CURL_SSLVERSION_TLSv1_2')) {
                    throw new \InvalidArgumentException('Invalid crypto_method request option: TLS 1.2 not supported by your version of cURL');
                }
                $conf[\CURLOPT_SSLVERSION] = \CURL_SSLVERSION_TLSv1_2;
            } elseif (defined('STREAM_CRYPTO_METHOD_TLSv1_3_CLIENT') && \STREAM_CRYPTO_METHOD_TLSv1_3_CLIENT === $options['crypto_method']) {
                if (!defined('CURL_SSLVERSION_TLSv1_3')) {
                    throw new \InvalidArgumentException('Invalid crypto_method request option: TLS 1.3 not supported by your version of cURL');
                }
                $conf[\CURLOPT_SSLVERSION] = \CURL_SSLVERSION_TLSv1_3;
            } else {
                throw new \InvalidArgumentException('Invalid crypto_method request option: unknown version provided');
            }
        }

        if (isset($options['cert'])) {
            $cert = $options['cert'];
            if (\is_array($cert)) {
                $conf[\CURLOPT_SSLCERTPASSWD] = $cert[1];
                $cert                         = $cert[0];
            }
            if (!\file_exists($cert)) {
                throw new \InvalidArgumentException("SSL certificate not found: {$cert}");
            }
            // OpenSSL(0.9.3及更高版本)也支持pkcs# 12编码文件的“P12”。
            // see https://curl.se/libcurl/c/CURLOPT_SSLCERTTYPE.html
            $ext = pathinfo($cert, \PATHINFO_EXTENSION);
            if (preg_match('#^(der|p12)$#i', $ext)) {
                $conf[\CURLOPT_SSLCERTTYPE] = strtoupper($ext);
            }
            $conf[\CURLOPT_SSLCERT] = $cert;
        }

        if (isset($options['ssl_key'])) {
            if (\is_array($options['ssl_key'])) {
                if (2 === \count($options['ssl_key'])) {
                    $sslKey                      = $options['ssl_key'][0];
                    $conf[\CURLOPT_SSLKEYPASSWD] = $options['ssl_key'][1];
                } else {
                    $sslKey = $options['ssl_key'][0];
                }
            }

            $sslKey = isset($sslKey) ? $sslKey : $options['ssl_key'];

            if (!\file_exists($sslKey)) {
                throw new \InvalidArgumentException("SSL private key not found: {$sslKey}");
            }
            $conf[\CURLOPT_SSLKEY] = $sslKey;
        }

        if (isset($options['progress'])) {
            $progress = $options['progress'];
            if (!\is_callable($progress)) {
                throw new \InvalidArgumentException('progress client option must be callable');
            }
            $conf[\CURLOPT_NOPROGRESS]       = false;
            $conf[\CURLOPT_PROGRESSFUNCTION] = static function ($resource, $downloadSize, $downloaded, $uploadSize, $uploaded) use ($progress) {
                $progress($downloadSize, $downloaded, $uploadSize, $uploaded);
            };
        }

        if (!empty($options['debug'])) {
            $conf[\CURLOPT_STDERR]  = Utils::debugResource($options['debug']);
            $conf[\CURLOPT_VERBOSE] = true;
        }
    }
    public function release(ResponseHandle $handle)
    {
        $resource = $handle->handle;
        unset($handle->handle);

        if (\count($this->handles) >= $this->maxHandles) {
            \curl_close($resource);
        } else {
            // 删除所有回调函数，因为它们可以保留引用，并且不会被curl_reset清除。
            // 由于某些原因，使用curl_setopt_array不起作用，因此逐个删除它们。
            \curl_setopt($resource, \CURLOPT_HEADERFUNCTION, null);
            \curl_setopt($resource, \CURLOPT_READFUNCTION, null);
            \curl_setopt($resource, \CURLOPT_WRITEFUNCTION, null);
            \curl_setopt($resource, \CURLOPT_PROGRESSFUNCTION, null);
            if (\PHP_VERSION_ID > 50500) {
                \curl_reset($resource);
            }
            $this->handles[] = $resource;
        }
    }

    /**
     * @return array<int|string, mixed>
     */
    private function getDefaultConf(ResponseHandle $handle)
    {
        $conf = [
            '_headers'              => $handle->request->getHeaders(),
            \CURLOPT_CUSTOMREQUEST  => $handle->request->getMethod(),
            \CURLOPT_URL            => (string) $handle->request->getUri()->withFragment(''),
            \CURLOPT_RETURNTRANSFER => false,
            \CURLOPT_HEADER         => false,
            \CURLOPT_CONNECTTIMEOUT => 300,
        ];

        if (\defined('CURLOPT_PROTOCOLS')) {
            $conf[\CURLOPT_PROTOCOLS] = \CURLPROTO_HTTP | \CURLPROTO_HTTPS;
        }

        $version = $handle->request->getProtocolVersion();
        if (1.1 == $version) {
            $conf[\CURLOPT_HTTP_VERSION] = \CURL_HTTP_VERSION_1_1;
        } elseif (2.0 == $version) {
            $conf[\CURLOPT_HTTP_VERSION] = \CURL_HTTP_VERSION_2_0;
        } else {
            $conf[\CURLOPT_HTTP_VERSION] = \CURL_HTTP_VERSION_1_0;
        }

        return $conf;
    }

    private function applyMethod(ResponseHandle $handle, array &$conf)
    {
        $body = $handle->request->getBody();
        $size = $body->getSize();

        if (null === $size || $size > 0) {
            $this->applyBody($handle->request, $handle->options, $conf);

            return;
        }

        $method = $handle->request->getMethod();
        if ('PUT' === $method || 'POST' === $method) {
            // See https://tools.ietf.org/html/rfc7230#section-3.3.2
            if (!$handle->request->hasHeader('Content-Length')) {
                $conf[\CURLOPT_HTTPHEADER][] = 'Content-Length: 0';
            }
        } elseif ('HEAD' === $method) {
            $conf[\CURLOPT_NOBODY] = true;
            unset(
                $conf[\CURLOPT_WRITEFUNCTION],
                $conf[\CURLOPT_READFUNCTION],
                $conf[\CURLOPT_FILE],
                $conf[\CURLOPT_INFILE]
            );
        }
    }

    private function applyBody(RequestInterface $request, array $options, array &$conf)
    {
        $size = $request->hasHeader('Content-Length')
            ? (int) $request->getHeaderLine('Content-Length')
            : null;

        // 如果消息体小于1MB，或者设置了[curl][body_as_string]请求值，则发送消息体为字符串。
        if ((null !== $size && $size < 1000000) || !empty($options['_body_as_string'])) {
            $conf[\CURLOPT_POSTFIELDS] = (string) $request->getBody();
            // 防止Content-Length协议头重复
            $this->removeHeader('Content-Length', $conf);
            $this->removeHeader('Transfer-Encoding', $conf);
        } else {
            $conf[\CURLOPT_UPLOAD] = true;
            if (null !== $size) {
                $conf[\CURLOPT_INFILESIZE] = $size;
                $this->removeHeader('Content-Length', $conf);
            }
            $body = $request->getBody();
            if ($body->isSeekable()) {
                $body->rewind();
            }
            $conf[\CURLOPT_READFUNCTION] = static function ($ch, $fd, $length) use ($body) {
                return $body->read($length);
            };
        }

        // 如果Expect标头不存在，防止curl添加
        if (!$request->hasHeader('Expect')) {
            $conf[\CURLOPT_HTTPHEADER][] = 'Expect:';
        }

        // cURL有时会默认添加一个内容类型。阻止curl的此行为。
        if (!$request->hasHeader('Content-Type')) {
            $conf[\CURLOPT_HTTPHEADER][] = 'Content-Type:';
        }
    }

    private function applyHeaders(ResponseHandle $handle, array &$conf)
    {
        foreach ($conf['_headers'] as $name => $values) {
            foreach ($values as $value) {
                $value = (string) $value;
                if ('' === $value) {
                    // cURL要求对空头文件采用特殊格式。
                    // https://github.com/guzzle/guzzle/issues/1882
                    $conf[\CURLOPT_HTTPHEADER][] = "$name;";
                } else {
                    $conf[\CURLOPT_HTTPHEADER][] = "$name: $value";
                }
            }
        }

        // 如果没有设置 Accept 协议，则删除 Accept 协议。
        if (!$handle->request->hasHeader('Accept')) {
            $conf[\CURLOPT_HTTPHEADER][] = 'Accept:';
        }
    }

    /**
     * 从协议头中删除一项。
     *
     * @param string $name    要删除的不区分大小写的协议头名称
     * @param array  $options 选项数组
     */
    private function removeHeader($name, array &$options)
    {
        foreach (\array_keys($options['_headers']) as $key) {
            if (!\strcasecmp($key, $name)) {
                unset($options['_headers'][$key]);

                return;
            }
        }
    }

    /**
     * @param ResponseHandle $handle
     * @param CurlFactory $factory Curl工厂，目前就一个，有需要再加
     * @return ResponseInterface|null
     */
    public static function finish(ResponseHandle $handle, CurlFactory $factory)
    {
        if (!$handle->response || $handle->errno) {
            self::finishError($handle, $factory);
        }

        // 如果存在并且没有错误，则返回响应。
        $factory->release($handle);

        // 如果可能的话，重置响应正文的指针位置。
        $body = $handle->response->getBody();
        if ($body->isSeekable()) {
            $body->rewind();
        }

        $response = $handle->response;

        if (isset($handle->responseMiddlewares)) {
            $response = $handle->responseMiddlewares->handler($response);
        }

        return $response;
    }

    private static function finishError(ResponseHandle $handle, CurlFactory $factory)
    {
        static $connectionErrors = [
            \CURLE_OPERATION_TIMEOUTED  => true,
            \CURLE_COULDNT_RESOLVE_HOST => true,
            \CURLE_COULDNT_CONNECT      => true,
            \CURLE_SSL_CONNECT_ERROR    => true,
            \CURLE_GOT_NOTHING          => true,
        ];

        $ctx = [
                'errno' => $handle->errno,
                'error' => \curl_error($handle->handle),
            ] + \curl_getinfo($handle->handle);
        if (\PHP_VERSION_ID >= 50500) {
            $ctx['appconnect_time'] = \curl_getinfo($handle->handle, \CURLINFO_APPCONNECT_TIME);
        }
        $ctx['curl_version'] = \curl_version()['version'];
        $factory->release($handle);

        if ($handle->createResponseException) {
            throw new RequestException('An error was encountered while creating the response', -1, $handle->createResponseException);
        }

        if ($handle->onHeadersException) {
            throw new RequestException('An error was encountered during the on_headers event', -1, $handle->onHeadersException);
        }

        if (isset($connectionErrors[$handle->errno])) {
            throw new ConnectException(
                \sprintf(
                    'cURL error %s: %s (see http://curl.haxx.se/libcurl/c/libcurl-errors.html)',
                    $ctx['errno'],
                    $ctx['error']
                ),
                $ctx['errno']
            );
        } else {
            throw new RequestException(
                \sprintf(
                    'cURL error %s: %s (see http://curl.haxx.se/libcurl/c/libcurl-errors.html)',
                    $ctx['errno'],
                    $ctx['error']
                ),
                $ctx['errno']
            );
        }
    }
}
