<?php

/**
 * WeEngine System
 *
 * (c) We7Team 2022 <https://www.w7.cc>
 *
 * This is not a free software
 * Using it under the license terms
 * visited https://www.w7.cc for more details
 */

namespace W7\Sdk\Module\Http;

use W7\Sdk\Module\Interfaces\UriInterface;

final class UriResolver
{
    /**
     * 从路径中移除点段并返回新路径。
     *
     * @link http://tools.ietf.org/html/rfc3986#section-5.2.4
     */
    public static function removeDotSegments($path)
    {
        if ('' === $path || '/' === $path) {
            return $path;
        }

        $results  = [];
        $segments = explode('/', $path);
        foreach ($segments as $segment) {
            if ('..' === $segment) {
                array_pop($results);
            } elseif ('.' !== $segment) {
                $results[] = $segment;
            }
        }

        $newPath = implode('/', $results);

        if ('/' === $path[0] && (!isset($newPath[0]) || '/' !== $newPath[0])) {
            // 如有必要，为"/... "等情况重新添加前面的斜线。
            $newPath = '/' . $newPath;
        } elseif ('' !== $newPath && ('.' === $segment || '..' === $segment)) {
            // 如有必要，请添加尾部斜杠
            // 如果newPath不是空的，那么$segment必须被设置，并且是foreach的最后一个段。
            $newPath .= '/';
        }

        return $newPath;
    }

    /**
     * 将相对URI转换为一个新的URI，并与base URI相对应进行解析
     *
     * @link http://tools.ietf.org/html/rfc3986#section-5.2
     */
    public static function resolve(UriInterface $base, UriInterface $rel)
    {
        if ('' === (string) $rel) {
            // 我们可以简单地返回相同的基础URI实例，用于这个相同的文件引用
            return $base;
        }

        if ('' != $rel->getScheme()) {
            return $rel->withPath(self::removeDotSegments($rel->getPath()));
        }

        if ('' != $rel->getAuthority()) {
            $targetAuthority = $rel->getAuthority();
            $targetPath      = self::removeDotSegments($rel->getPath());
            $targetQuery     = $rel->getQuery();
        } else {
            $targetAuthority = $base->getAuthority();
            if ('' === $rel->getPath()) {
                $targetPath  = $base->getPath();
                $targetQuery = '' != $rel->getQuery() ? $rel->getQuery() : $base->getQuery();
            } else {
                if ('/' === $rel->getPath()[0]) {
                    $targetPath = $rel->getPath();
                } else {
                    if ('' != $targetAuthority && '' === $base->getPath()) {
                        $targetPath = '/' . $rel->getPath();
                    } else {
                        $lastSlashPos = strrpos($base->getPath(), '/');
                        if (false === $lastSlashPos) {
                            $targetPath = $rel->getPath();
                        } else {
                            $targetPath = substr($base->getPath(), 0, $lastSlashPos + 1) . $rel->getPath();
                        }
                    }
                }
                $targetPath  = self::removeDotSegments($targetPath);
                $targetQuery = $rel->getQuery();
            }
        }

        return new Uri(Uri::composeComponents(
            $base->getScheme(),
            $targetAuthority,
            $targetPath,
            $targetQuery,
            $rel->getFragment()
        ));
    }

    /**
     * 返回目标URI，作为基础URI的相对引用。
     *
     * @param UriInterface $base
     * @param UriInterface $target
     * @return UriInterface
     */
    public static function relativize(UriInterface $base, UriInterface $target)
    {
        if ('' !== $target->getScheme() && ($base->getScheme() !== $target->getScheme() || '' === $target->getAuthority() && '' !== $base->getAuthority())
        ) {
            return $target;
        }

        if (Uri::isRelativePathReference($target)) {
            return $target;
        }

        if ('' !== $target->getAuthority() && $base->getAuthority() !== $target->getAuthority()) {
            return $target->withScheme('');
        }

        $emptyPathUri = $target->withScheme('')->withPath('')->withUserInfo('')->withPort(null)->withHost('');

        if ($base->getPath() !== $target->getPath()) {
            return $emptyPathUri->withPath(self::getRelativePath($base, $target));
        }

        if ($base->getQuery() === $target->getQuery()) {
            return $emptyPathUri->withQuery('');
        }

        if ('' === $target->getQuery()) {
            $segments = explode('/', $target->getPath());
            /** @var string $lastSegment */
            $lastSegment = end($segments);

            return $emptyPathUri->withPath('' === $lastSegment ? './' : $lastSegment);
        }

        return $emptyPathUri;
    }

    private static function getRelativePath(UriInterface $base, UriInterface $target)
    {
        $sourceSegments = explode('/', $base->getPath());
        $targetSegments = explode('/', $target->getPath());
        array_pop($sourceSegments);
        $targetLastSegment = array_pop($targetSegments);
        foreach ($sourceSegments as $i => $segment) {
            if (isset($targetSegments[$i]) && $segment === $targetSegments[$i]) {
                unset($sourceSegments[$i], $targetSegments[$i]);
            } else {
                break;
            }
        }
        $targetSegments[] = $targetLastSegment;
        $relativePath     = str_repeat('../', count($sourceSegments)) . implode('/', $targetSegments);

        if ('' === $relativePath || false !== strpos(explode('/', $relativePath, 2)[0], ':')) {
            $relativePath = "./$relativePath";
        } elseif ('/' === $relativePath[0]) {
            if ('' != $base->getAuthority() && '' === $base->getPath()) {
                $relativePath = ".$relativePath";
            } else {
                $relativePath = "./$relativePath";
            }
        }

        return $relativePath;
    }

    private function __construct()
    {
    }
}
