<?php

/**
 * WeEngine System
 *
 * (c) We7Team 2022 <https://www.w7.cc>
 *
 * This is not a free software
 * Using it under the license terms
 * visited https://www.w7.cc for more details
 */

namespace W7\Sdk\Module\Support;

use W7\Sdk\Module\Http\PumpStream;
use W7\Sdk\Module\Http\Stream;
use W7\Sdk\Module\Http\Uri;
use W7\Sdk\Module\Interfaces\StreamInterface;
use W7\Sdk\Module\Interfaces\UriInterface;

class Utils
{
    /**
     * 安全地获取一个指定流的内容。
     *
     * 当 stream_get_contents 失败时，PHP 通常会提出一个警告。此函数增加了一个错误处理程序，检查错误并抛出一个异常。
     *
     * @param resource $stream
     *
     * @throws \RuntimeException
     */
    public static function tryGetContents($stream)
    {
        $ex = null;
        set_error_handler(static function ($errno, $errStr) use (&$ex) {
            $ex = new \RuntimeException(sprintf(
                'Unable to read stream contents: %s',
                $errStr
            ));

            return true;
        });

        try {
            /** @var string|false $contents */
            $contents = stream_get_contents($stream);

            if (false === $contents) {
                $ex = new \RuntimeException('Unable to read stream contents');
            }
        } catch (\RuntimeException $e) {
            $ex = new \RuntimeException(sprintf(
                'Unable to read stream contents: %s',
                $e->getMessage()
            ), 0, $e);
        }

        restore_error_handler();

        if ($ex) {
            throw $ex;
        }

        return $contents;
    }

    /**
     * 根据输入类型创建一个新的流。
     *
     * @param resource|string|int|float|bool|StreamInterface|callable|\Iterator|null $resource
     * @param  array{size?: int, metadata?: array}  $options
     *
     * @return object|PumpStream|Stream|StreamInterface
     */
    public static function streamFor($resource = '', array $options = [])
    {
        if (is_scalar($resource)) {
            $stream = self::tryFopen('php://temp', 'r+');
            if ('' !== $resource) {
                fwrite($stream, (string)$resource);
                fseek($stream, 0);
            }
            return new Stream($stream, $options);
        }

        switch (gettype($resource)) {
            case 'resource':
                $meta = \stream_get_meta_data($resource);
                $uri  = (isset($meta['uri'])) ? $meta['uri'] : '';
                if ('php://input' === $uri) {
                    $stream = self::tryFopen('php://temp', 'w+');
                    stream_copy_to_stream($resource, $stream);
                    fseek($stream, 0);
                    $resource = $stream;
                }
                return new Stream($resource, $options);
            case 'object':
                /** @var object $resource */
                if ($resource instanceof StreamInterface) {
                    return $resource;
                } elseif ($resource instanceof \Iterator) {
                    return new PumpStream(function () use ($resource) {
                        if (!$resource->valid()) {
                            return false;
                        }
                        $result = $resource->current();
                        $resource->next();
                        return $result;
                    }, $options);
                } elseif (method_exists($resource, '__toString')) {
                    return self::streamFor((string) $resource, $options);
                }
                break;
            case 'NULL':
                return new Stream(self::tryFopen('php://temp', 'r+'), $options);
        }

        throw new \InvalidArgumentException('Invalid resource type: ' . gettype($resource));
    }

    /**
     * 使用文件名安全地打开一个PHP流资源。
     *
     *
     * @param string $filename 要打开的文件名
     * @param string $mode     打开文件的模式
     *
     * @return resource
     *
     * @throws \RuntimeException
     */
    public static function tryFopen($filename, $mode)
    {
        $ex = null;
        set_error_handler(static function ($errno, $errStr) use ($filename, $mode, &$ex) {
            $ex = new \RuntimeException(sprintf(
                'Unable to open "%s" using mode "%s": %s',
                $filename,
                $mode,
                $errStr
            ));

            return true;
        });

        /** @var resource $handle */
        $handle = fopen($filename, $mode);

        restore_error_handler();

        if ($ex) {
            throw $ex;
        }

        return $handle;
    }

    /**
     * 将一个流的内容复制到一个字符串中，直到读完给定的字节数。
     *
     * @param StreamInterface $stream 要读取的流
     * @param int             $maxLen 要读取的最大字节数。通过-1来读取整个流
     *
     * @return string
     *
     * @throws \RuntimeException
     */
    public static function copyToString(StreamInterface $stream, $maxLen = -1)
    {
        $buffer = '';

        if (-1 === $maxLen) {
            while (!$stream->eof()) {
                $buf = $stream->read(1048576);
                if ('' === $buf) {
                    break;
                }
                $buffer .= $buf;
            }
            return $buffer;
        }

        $len = 0;
        while (!$stream->eof() && $len < $maxLen) {
            $buf = $stream->read($maxLen - $len);
            if ('' === $buf) {
                break;
            }
            $buffer .= $buf;
            $len = strlen($buffer);
        }

        return $buffer;
    }

    /**
     * 将一个协议头行数组解析为一个协议头标题的关联数组。
     *
     * @param iterable $lines 协议头的字符串格式如下： "名称： 值"
     *
     * @return array
     */
    public static function headersFromLines($lines)
    {
        $headers = [];

        foreach ($lines as $line) {
            $parts                       = \explode(':', $line, 2);
            $headers[\trim($parts[0])][] = isset($parts[1]) ? \trim($parts[1]) : null;
        }

        return $headers;
    }

    /**
     * 创建一个小写头名称的关联数组，值与实际头的大小写一致。
     *
     * @return array
     */
    public static function normalizeHeaderKeys(array $headers)
    {
        $result = [];
        foreach (\array_keys($headers) as $key) {
            $result[\strtolower($key)] = $key;
        }

        return $result;
    }

    /**
     * 为给定的值返回一个UriInterface。
     *
     *
     * @param string|UriInterface $uri
     *
     * @throws \InvalidArgumentException
     */
    public static function uriFor($uri)
    {
        if ($uri instanceof UriInterface) {
            return $uri;
        }

        if (is_string($uri)) {
            return new Uri($uri);
        }

        throw new \InvalidArgumentException('URI must be a string or UriInterface');
    }

    /**
     * 如果提供的主机与任何一个无代理区域相匹配，则返回真。
     *
     *
     * @param string   $host         主机检查模式
     * @param string[] $noProxyArray 宿主模式数组
     *
     * @throws \InvalidArgumentException
     */
    public static function isHostInNoProxy($host, array $noProxyArray)
    {
        if (0 === \strlen($host)) {
            throw new \InvalidArgumentException('Empty host provided');
        }

        $hosts = \explode(':', $host, 2);
        $host  = $hosts[0];

        foreach ($noProxyArray as $area) {
            if ('*' === $area) {
                return true;
            }

            if (empty($area)) {
                continue;
            }

            if ($area === $host) {
                return true;
            }
            $area = '.' . \ltrim($area, '.');
            if (\substr($host, -\strlen($area)) === $area) {
                return true;
            }
        }

        return false;
    }

    /**
     * 根据提供的变量返回一个调试流。
     *
     * @param mixed $value
     *
     * @return resource
     */
    public static function debugResource($value = null)
    {
        if (\is_resource($value)) {
            return $value;
        }
        if (\defined('STDOUT')) {
            return \STDOUT;
        }

        return Utils::tryFopen('php://output', 'w');
    }
}
